#include "ConfigDialog.h"
#include "ui_ConfigDialog.h"
#include "AfcFltConfig.h"
#include "lqmeas/devs/LTR/modules/LTR35/LTR35.h"

ConfigDialog::ConfigDialog(QWidget *parent, const AfcFltConfig &cfg, LQMeas::LTR35 *pltr35) :
    QDialog{parent},
    ui{new Ui::ConfigDialog},
    m_cfg{cfg},
    m_pltr35{pltr35} {

    ui->setupUi(this);

    m_chsK.append(ui->k_ch1);
    m_chsK.append(ui->k_ch2);
    m_chsK.append(ui->k_ch3);
    m_chsK.append(ui->k_ch4);
    m_chsK.append(ui->k_ch5);
    m_chsK.append(ui->k_ch6);
    m_chsK.append(ui->k_ch7);
    m_chsK.append(ui->k_ch8);

    showConfig();

    connect(ui->isManual, &QCheckBox::stateChanged, this, &ConfigDialog::setManual);
    connect(ui->afcCorEn, &QCheckBox::stateChanged, this, &ConfigDialog::setEnabled);
    connect(ui->f0, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &ConfigDialog::setF0);
    connect(ui->fd0, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
            this, &ConfigDialog::setFD0);
    for (QDoubleSpinBox *db : m_chsK) {
        connect(db, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
                this, &ConfigDialog::setK);
    }
}

ConfigDialog::~ConfigDialog() {
    delete ui;
}

void ConfigDialog::fillConfig(AfcFltConfig &cfg) {
    cfg = m_cfg;
}

void ConfigDialog::showConfig() {
    ui->isManual->setChecked(m_cfg.isManual());
    ui->afcCorEn->setChecked(m_cfg.enabled());

    ui->manualCfg->setEnabled(m_cfg.isManual());

    if (m_cfg.isManual()) {
        ui->f0->setValue(m_cfg.f0());
        ui->fd0->setValue(m_cfg.fd0());
        for (int ch = 0; ch  < AfcFltConfig::ch_cnt; ch++) {
            m_chsK.at(ch)->setValue(m_cfg.k(ch));
        }
    } else {
        if (m_pltr35){
            const TLTR35_AFC_COEF &afcCoef = m_pltr35->rawHandle()->ModuleInfo.CbrAfcCoef;
            ui->f0->setValue(afcCoef.SigFreq);
            ui->fd0->setValue(afcCoef.Fd);
            for (int ch = 0; ch  < AfcFltConfig::ch_cnt; ch++) {
                m_chsK.at(ch)->setValue(afcCoef.K[ch]);
            }
        } else {
            ui->f0->setValue(AfcFltConfig::defaultF0());
            ui->fd0->setValue(AfcFltConfig::defaultFd0());
            for (int ch = 0; ch  < AfcFltConfig::ch_cnt; ch++) {
                m_chsK.at(ch)->setValue(AfcFltConfig::defaultK());
            }
        }
    }
}

void ConfigDialog::setManual() {
    if (!m_ui_update) {
        m_ui_update = true;
        m_cfg.setManual(ui->isManual->isChecked());
        showConfig();
        m_ui_update = false;
    }
}

void ConfigDialog::setEnabled() {
    if (!m_ui_update) {
        m_ui_update = true;
        m_cfg.setEnabled(ui->afcCorEn->isChecked());
        m_ui_update = false;
    }
}

void ConfigDialog::setF0() {
    if (!m_ui_update && m_cfg.isManual()) {
        m_ui_update = true;
        m_cfg.setEnabled(ui->f0->value());
        m_ui_update = false;
    }
}

void ConfigDialog::setFD0() {
    if (!m_ui_update && m_cfg.isManual()) {
        m_ui_update = true;
        m_cfg.setEnabled(ui->fd0->value());
        m_ui_update = false;
    }
}

void ConfigDialog::setK() {
    if (!m_ui_update && m_cfg.isManual()) {
        m_ui_update = true;
        QDoubleSpinBox *db {qobject_cast<QDoubleSpinBox*>(sender())};
        if (db) {
            int ch_num = m_chsK.indexOf(db);
            if (ch_num >= 0) {
                m_cfg.setK(ch_num, db->value());
            }
        }
        m_ui_update = false;
    }
}
