#include "LedTextDelegate.h"
#include <QPainter>
#include <QApplication>

LedTextDelegate::LedTextDelegate(QObject *parent, DelegateIndexSelector *paintSelector) : LedDelegate(parent, paintSelector) {

}

void LedTextDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    if (!paintEnabled(index)) {
        QStyledItemDelegate::paint(painter, option, index);
    } else {

        int ledSize = qMin(option.rect.width() - 20, option.rect.height());
        QRect ledRect(option.rect.left(), option.rect.top(), ledSize, ledSize);
        paintLed(painter, option, ledRect, index);

        QRect textRect = QRect(option.rect.left() + ledSize, option.rect.top(), option.rect.width() - ledSize, option.rect.height());
        QString text = index.data(Qt::DisplayRole).toString();


        QVariant bgBrush = index.data(Qt::BackgroundRole);
        if (bgBrush.isValid()) {
            painter->fillRect(textRect, bgBrush.value<QBrush>());
        } else if (option.state & QStyle::State_Selected) {
            painter->fillRect(textRect, option.palette.highlight());
        }
        QApplication::style()->drawItemText(painter, textRect, Qt::AlignVCenter | Qt::AlignLeft,
                                            option.palette, true, text,
                                            (option.state & QStyle::State_Selected ? QPalette::HighlightedText : QPalette::Text));
    }
}
