#ifndef LQMEAS_DEVICECONFIGIFACE_H
#define LQMEAS_DEVICECONFIGIFACE_H

#include <QObject>
class QJsonObject;

namespace LQMeas {
    class DeviceTypeInfo;
    class Device;
    class DeviceConfig;

    class DeviceConfigIface : public QObject {
        Q_OBJECT
    public:

        virtual ~DeviceConfigIface();
        /* сохранение параметров конфигурации в объект JSON */
        void save(QJsonObject &cfgObj) const;
        /* загрузка параметров конфигурации из объекта JSON */
        void load(const QJsonObject &cfgObj);

        void init(DeviceConfig *cfg);

        DeviceConfig *devConfig() const {return  m_devCfg;}
    public Q_SLOTS:
        void update();
    Q_SIGNALS:
        void configChanged();
        void updated();
    protected:
        DeviceConfigIface();

        virtual QString cfgIfaceName() const = 0;

        void notifyConfigChanged();

        virtual void protInit(DeviceConfig *cfg) {Q_UNUSED(cfg) }
        virtual void protSave(QJsonObject &cfgObj) const { Q_UNUSED(cfgObj) }
        virtual void protLoad(const QJsonObject &cfgObj) { Q_UNUSED(cfgObj) }
        virtual void protUpdate() {}
        virtual void protUpdateInfo(const DeviceTypeInfo *info) { Q_UNUSED(info) }
        virtual void protUpdateDevice(const Device *device) { Q_UNUSED(device) }
        virtual void protUpdateParentConfig(const DeviceConfig *parentCfg) {Q_UNUSED(parentCfg)}
    private:
        friend class DeviceConfig;
        DeviceConfig *m_devCfg {nullptr};
    };
}

#endif // LQMEAS_DEVICECONFIGIFACE_H
