#ifndef LQMEAS_LTR35OUTCONFIG_H
#define LQMEAS_LTR35OUTCONFIG_H

#include "lqmeas/ifaces/out/DevOutSyncConfig.h"

namespace LQMeas {
    class LTR35Config;

    class LTR35OutConfig: public DevOutSyncConfig {
        Q_OBJECT
    public:
        enum class HardSrcOutput {
            Sin = 0,
            Cos
        };

        enum class OutDataFormat {
            Unspecified = 0, /* Формат явно не определен. Определятся
                                         автоматически по режиму */
            Format_20,
            Format_24
        };

        enum class OutStdGenFreq {
            Freq_192K = 0,
            Freq_96K,
            Freq_48K
        };


        bool outSyncExternalStart() const override;

        bool outSyncGenModeEnabled() const {return params.syncMode;}
        void outSetSyncGenModeEnabled(bool en);

        void outSyncSetManualGenFreq(double val);
        double outSyncManualGenFreq() const {return params.outFreq.manualVal;}

        void outSyncSetStdGenFreq(OutStdGenFreq freq);
        OutStdGenFreq outSyncStdGenFreq() const {return params.outFreq.stdVal;}

        void outSyncSetManualGenFreqEnabled(bool en);
        bool outSyncManualGenFreqEnabled() const {return params.outFreq.manualEn;}

        double outSyncConfiguredGenFreq() const;
        double outSyncGenFreq() const override {return results.OutFreq;}

        void outDacSetChRangeNum(int ch, int range);
        int outDacChRangeNum(int ch) const override {return params.Chs[ch].range;}

        bool outDacChSyncModeEnabled(int ch) const  override { Q_UNUSED(ch) return params.syncMode;}
        bool outDigChSyncModeEnabled(int ch) const override { Q_UNUSED(ch) return params.syncMode;}


        void outDacChSetHardSourceNumber(int ch, int src_num);
        void outDacChSetHardSourceOutput(int ch, HardSrcOutput out);

        int  outDacChHardSourceNumber(int ch) const {return params.Chs[ch].hard_src_num;}
        HardSrcOutput  outDacChHardSourceOutput(int ch) const {return params.Chs[ch].hard_src_out;}


        OutDataFormat outDacDataFormat() const {return params.dacDataFmt;}
        void outDacSetDataFormat(OutDataFormat fmt);

        bool outSyncEchoEnabled() const override {return params.echoEnabled;}
        void outSyncSetEchoEnabled(bool en);

        int outSyncEchoChannel() const {return params.echoChannel;}
        void outSyncSetEchoChannel(int ch);

        bool outDacGenRawCodeMode() const {return  params.dacRawCodeMode;}
        void outDacSetGenRawCodeMode(bool en);

        bool outDacAfcCorEnabled() const {return params.dacAfcCorEn;}
        void outDacSetAfcCorEnabled(bool en);

        bool outSyncPeriodicStatusEnabled() const {return params.periodicStatusEn;}
        void outSyncSetPeriodicStatusEnabled(bool en);

        static constexpr bool defaultOutSyncManualGenFreqEnabled() {return false;}
        static constexpr double defaultOutGenFreqValue() {return 192000;}
        static constexpr OutStdGenFreq defaultOutSyncStdGenFreq() {return OutStdGenFreq::Freq_192K;}

        static const int max_dac_ch_cnt = 8;

        static int typeStdGenFreqNum(OutStdGenFreq freq) { return static_cast<int>(freq);}
        static double typeStdGenFreqVal(OutStdGenFreq freq);
    protected:
        void protSave(QJsonObject &cfgObj) const override;
        void protLoad(const QJsonObject &cfgObj) override;
        void protUpdate() override;
    private:


        friend class LTR35Config;
        LTR35OutConfig(const LTR35Config &ltr35cfg, const DevOutInfo &info);
        LTR35OutConfig(const LTR35Config &ltr35cfg, const LTR35OutConfig &outCfg);


        struct {
            struct {
                int range;
                int  hard_src_num;
                HardSrcOutput hard_src_out;
            } Chs[max_dac_ch_cnt];

            struct {
                bool manualEn;
                double manualVal;
                OutStdGenFreq stdVal;
            } outFreq;

            OutDataFormat dacDataFmt;
            bool syncMode;
            bool dacRawCodeMode;
            bool dacAfcCorEn;
            bool periodicStatusEn;

            bool echoEnabled;
            int echoChannel;
        } params;


        struct {
            double OutFreq;
        } results;

        const LTR35Config *m_ltr35cfg;
    };
}

Q_DECLARE_METATYPE(LQMeas::LTR35OutConfig::HardSrcOutput);
Q_DECLARE_METATYPE(LQMeas::LTR35OutConfig::OutDataFormat);
Q_DECLARE_METATYPE(LQMeas::LTR35OutConfig::OutStdGenFreq);

#endif // LQMEAS_LTR35OUTCONFIG_H

