#ifndef LQMEAS_LTR35TYPEINFO_H
#define LQMEAS_LTR35TYPEINFO_H

#include "lqmeas/devs/DeviceTypeInfo.h"
#include "lqmeas/ifaces/out/DevOutInfo.h"
#include "lqmeas/ifaces/in/dig/DevInDigInfo.h"

namespace LQMeas {
    class LTR35TypeInfo : public DeviceTypeInfo, public DevOutInfo, public DevInDigInfo {
    public:
        static const QString &name();
        QString deviceTypeName() const override {return name();}
        QString deviceModificationName() const override;

        static const LTR35TypeInfo &defaultInfo();
        static const LTR35TypeInfo &typeInfoMod1();
        static const LTR35TypeInfo &typeInfoMod1_8();
        static const LTR35TypeInfo &typeInfoMod1_4();
        static const LTR35TypeInfo &typeInfoMod2();
        static const LTR35TypeInfo &typeInfoMod2_4();
        static const LTR35TypeInfo &typeInfoMod2_8();
        static const LTR35TypeInfo &typeInfoMod3();


        /* ----------------- DevOutInfo -------------------------------------*/
        const DevOutInfo *out() const override {return static_cast<const DevOutInfo *>(this);}
        const DevInDigInfo *digin() const override {return static_cast<const DevInDigInfo *>(this);}

        bool outDacSyncSupport() const override {return outDacChannelsCnt() != 0;}
        bool outDigSyncSupport() const override {return true;}

        bool outDacAsyncSupport() const override {return outDacChannelsCnt() != 0;}
        bool outDigAsyncSupport() const override {return true;}

        bool outDacSyncModeCfgPerCh() const override {return false;}

        bool outSyncRamModeIsConfigurable() const override {return true;}
        bool outSyncDacChGenModeIsConfigurable() const override {return true;}
        bool outSyncDigChGenModeIsConfigurable() const override {return false;}


        double outSyncGenFreqMax() const override;
        bool outSyncPresetSupport() const override {return false;}

        int outDacChannelsCnt() const override {return m_dac_ch_cnt;}

        int inDigChannelsCnt() const override {return 2;}

        /* в данном случае возвращаются модификации, которые реально существуют */
        QList<const DeviceTypeInfo *> modificationList() const override;

        virtual int modificationNum() const = 0;

        static double typeStdGenFreqVal(int idx);
        static int typeStdGenFreqsCnt();
    protected:
        LTR35TypeInfo(int dac_ch_cnt);
    private:
        int m_dac_ch_cnt;
    };
}

#endif // LQMEAS_LTR35TYPEINFO_H
