#include "DevAdcConfig.h"
#include "lqmeas/devs/DeviceTypeInfo.h"
namespace LQMeas {

    DevAdcConfig::DevAdcConfig(const DevAdcInfo &info) : m_adc_info{&info} {

    }

    DevAdcConfig::DevAdcConfig(const DevAdcConfig &cfg) {
        m_adc_info = cfg.adcInfo();
        m_ch_indexes = cfg.m_ch_indexes;
    }

    DevAdcConfig::~DevAdcConfig() {

    }

    double DevAdcConfig::adcChFreq() const {
        double ch_freq  = adcFreq();
        if (m_adc_info->adcType() == DevAdcInfo::AdcType::Sequential) {
            int enChCnt = adcEnabledChCnt();
            if (enChCnt) {
                ch_freq/=enChCnt;
            }
        }
        return ch_freq;
    }

    double DevAdcConfig::adcChTimeShift(int ch) const {
        return m_adc_info->adcType() == DevAdcInfo::AdcType::Sequential ?
                    static_cast<double>(adcChIdx(ch))/adcFreq() : 0;
    }

    int DevAdcConfig::adcChNum(int idx) const {
        return idx < m_ch_indexes.size() ? m_ch_indexes[idx] : -1;
    }

    int DevAdcConfig::adcChIdx(int ch) const {
        return m_ch_indexes.indexOf(ch);
    }

    double DevAdcConfig::adcDefaultFreq() const     {
        return m_adc_info->adcFreqDefault();
    }


    void DevAdcConfig::notifyChEnableStateChanged(int ch) {
        Q_EMIT adcChEnableStateChanged(ch, adcChEnabled(ch));
        Q_EMIT adcChConfigChanged(ch);
    }

    void DevAdcConfig::notifyChRangeChanged(int ch) {
        Q_EMIT adcChRangeChanged(ch, adcChRangeMinVal(ch), adcChRangeMaxVal(ch));
        Q_EMIT adcChConfigChanged(ch);
    }

    void DevAdcConfig::notifyChModeChanged(int ch) {
        Q_EMIT adcChModeChanged(ch);
        Q_EMIT adcChConfigChanged(ch);
    }

    void DevAdcConfig::notifyChFreqChanged(int ch) {
        Q_EMIT adcChConfiguredFreqChanged(ch);
        Q_EMIT adcChConfigChanged(ch);
    }

    void DevAdcConfig::notifyAdcFreqChanged() {
        Q_EMIT adcConfiguredFreqChanged();
    }

    void DevAdcConfig::notifyChConfigChanged(int ch)     {
        Q_EMIT adcChConfigChanged(ch);
    }

    void DevAdcConfig::protUpdate() {
        m_ch_indexes.clear();
        for (int ch =0; ch < m_adc_info->adcChannelsCnt(); ++ch) {
            if (adcChEnabled(ch)) {
                m_ch_indexes.append(ch);
            }
        }
    }

    void DevAdcConfig::protUpdateInfo(const DeviceTypeInfo *info)  {
        m_adc_info = info->adc();
        update();
    }

}
